## Example 2: Modals

<div class="grid">

```typescript
import React, { MouseEventHandler, PureComponent, ReactNode } from 'react';
import ReactDOM from 'react-dom';
import styled, { css } from 'styled-components';

import { H4 } from '../Typography';

import Icon from 'components/Icon';
import isEventEscapeKey from 'helpers/isEventEscapeKey';
import theme from 'theme';

export const CloseIcon = styled(Icon).attrs({
  size: 20,
})`
  color: ${theme.colours.primary500};
  transition: color 0.3s ease;
  position: absolute;
  right: 30px;
  top: 30px;
  z-index: 1;

  &[disabled] {
    opacity: 0.6;
  }

  &:not([disabled]) {
    cursor: pointer;
  }
`;

export const ModalOverlay = styled.div`
  top: 0;
  left: 0;
  position: fixed;
  display: flex;
  justify-content: center;
  align-items: center;
  height: 100%;
  width: 100%;
  z-index: 1000;
  background-color: rgba(0, 0, 0, 0.4);
`;

export const ModalContent = styled.div<StyleProps>`
  min-width: 500px;
  background-color: white;
  border-radius: 4px;
  padding: 30px;
  position: relative;
  line-height: 1.43;
  max-height: 80%;

  ${({ scrollable }) =>
    scrollable
      ? css`
          overflow-y: auto;
        `
      : ''};
`;

export type StyleProps = {
  scrollable?: boolean;
};

export type Props = StyleProps & {
  children: ReactNode;
  onClose: MouseEventHandler<HTMLDivElement | HTMLButtonElement>;
  className?: string;
  disabled?: boolean;
};

class ModalContainer extends PureComponent<Props> {
  componentDidMount() {
    document.addEventListener('keydown', this.onEscapePress);
  }

  componentWillUnmount() {
    document.removeEventListener('keydown', this.onEscapePress);
  }

  overlayClicked: MouseEventHandler<HTMLDivElement> = (e) => {
    e.stopPropagation();
    if (!this.props.disabled && e.target === e.currentTarget) {
      this.props.onClose(e);
    }
  };

  onEscapePress: EventListenerOrEventListenerObject = (e) => {
    e.stopPropagation();
    if (!this.props.disabled && isEventEscapeKey(e as KeyboardEvent)) {
      this.props.onClose(e as any);
    }
  };

  onClose: MouseEventHandler<HTMLDivElement> = (e) => {
    e.stopPropagation();
    if (!this.props.disabled) {
      this.props.onClose(e);
    }
  };

  render() {
    const modal = (
      <ModalOverlay onClick={this.overlayClicked}>
        <ModalContent className={this.props.className} scrollable={this.props.scrollable}>
          <CloseIcon onClick={this.onClose} icon="close" />
          {this.props.children}
        </ModalContent>
      </ModalOverlay>
    );

    // defaulting to document.body mostly for the benefit of tests, which don't have a #app-container element.
    const rootElement = document.getElementById('app-container') || document.body;

    return ReactDOM.createPortal(modal, rootElement);
  }
}


export const ModalContents = styled.div`
  display: flex;
  flex-direction: column;
`;

export const ModalHeader = styled(H4)`
  margin-bottom: 15px;
`;

export const ModalBody = styled.div`
  font-size: 14px;
  line-height: 25px;
`;

export const ModalFooter = styled.div`
  width: 100%;
  display: flex;
  justify-content: flex-end;
  align-items: center;
  margin-top: 15px;

  & > {
    margin-left: 20px;
  }
`;

type Props = {
  visible: boolean;
  header: ReactNode;
  body: ReactNode;
  footer?: ReactNode;
  loading?: boolean;
  onClose: () => void;
};

class Modal extends PureComponent<Props> {
  render() {
    const { visible, header, body, footer, onClose, loading } = this.props;

    if (!visible) {
      return null;
    }

    return (
      <ModalContainer onClose={onClose} disabled={loading} scrollable={true}>
        <ModalContents>
          <ModalHeader>{header}</ModalHeader>
          <ModalBody>{body}</ModalBody>
          <ModalFooter>{footer}</ModalFooter>
        </ModalContents>
      </ModalContainer>
    );
  }
}
```

<div>

<div data-react-root="modal"></div>

* 171 lines of code
* Requires state updates to display
* Separate component just to manage the backdrop
* How does a screen reader announce that the modal is displayed? (hint: it doesn't)

</div>

</div>